function resetCounters()
    hbCount = HBView.getHBCount()

    rowsCount = (hbCount + hbPerRow - 1) // hbPerRow
end

function init()
    font = Gfx.loadFont("assets/fonts/OpenSans-Regular.ttf", 16)
    winTitleFont = Gfx.loadFont("assets/fonts/OpenSans-Regular.ttf", 22)

    screensize = Gui.getScreenSize()
    if screensize.x == 640 then
        hbPerRow = 4
    else
        hbPerRow = 5
    end

    rowsScroll = 0
    rowsPerPage = 4
    coverWidth = 128
    coverHeight = 48
    selectorBorder = 3
    gridGap = 10
    selectedHB = 0
    prevSelectedHB = 0
    textScroll = 0
    scrollTextGap = 32
    scrollTextTimeRef = 0
    scrollTextTime = 50
    HBView.setCoverSize(coverWidth, coverHeight)
    resetCounters()
end

function addHBCB()
    resetCounters()
end

function draw(onFocus)
    Gfx.print(winTitleFont, 0, 0, "Homebrews")
    Gfx.translate(0, 30)
    local winWidth = hbPerRow * coverWidth + (hbPerRow - 1) * gridGap + 2 * selectorBorder
    local winHeight = rowsPerPage * (coverHeight + 28) + (rowsPerPage - 1) * gridGap + 2 * selectorBorder
    Gfx.pushScissorBox(winWidth, winHeight)
    Gfx.translate(0, -rowsScroll * (coverHeight + gridGap + 28))
    Gfx.translate(selectorBorder, selectorBorder)
    for i = 0, (hbCount - 1) do
        local x = (i % hbPerRow) * (coverWidth + gridGap)
        local y = (i // hbPerRow) * (coverHeight + gridGap + 28)
        if i == selectedHB and onFocus then
            Gfx.drawRectangle(x - selectorBorder, y - selectorBorder, coverWidth + 2 * selectorBorder, coverHeight + 2 * selectorBorder, Gfx.RGBA8(0x25, 0xcb, 0xf9, 0xff))
        end
        HBView.drawHBCover(x, y, i)

        if selectedHB ~= prevSelectedHB then
            scrollTextTimeRef = Time.getms()
            prevSelectedHB = selectedHB
            textScroll = 0
        end

        Gfx.pushMatrix()
        Gfx.translate(x, y + coverHeight + 8)
        Gfx.pushScissorBox(coverWidth, 20)
        if i == selectedHB and onFocus then
            Gfx.translate(-textScroll, 0)
            local textWidth = Gfx.print(font, 0, 0, HBView.getHBName(i))
            if textWidth > coverWidth then
                Gfx.print(font, textWidth + scrollTextGap, 0, HBView.getHBName(i))
                textScroll = (Time.getms() - scrollTextTimeRef) // scrollTextTime
                if textScroll > textWidth + scrollTextGap then
                    textScroll = textScroll - (textWidth + scrollTextGap)
                    scrollTextTimeRef = Time.getms() - textScroll * scrollTextTime
                end
            end
        else
            Gfx.print(font, 0, 0, HBView.getHBName(i))
        end
        Gfx.popScissorBox()
        Gfx.popMatrix()
    end
    Gfx.popScissorBox()
end

function handleInputs()
    local down = Pad.gendown(0)

    if down.BUTTON_RIGHT then
        selectedHB = selectedHB + 1
    elseif down.BUTTON_LEFT then
        selectedHB = selectedHB - 1
    elseif down.BUTTON_UP then
        selectedHB = selectedHB - hbPerRow
    elseif down.BUTTON_DOWN then
        selectedHB = selectedHB + hbPerRow
    end

    --Boundary checks
    if selectedHB < 0 then
        selectedHB = 0
    elseif selectedHB >= hbCount then
        selectedHB = hbCount - 1;
    end

    --Scroll
    if rowsCount > rowsPerPage then
        local curRow = (selectedHB // hbPerRow) - rowsScroll
        if curRow < (rowsPerPage // 2 - 1) and rowsScroll > 0 and down.BUTTON_UP then
            rowsScroll = rowsScroll - 1
        elseif curRow > (rowsPerPage // 2) and (rowsScroll + rowsPerPage) < rowsCount and down.BUTTON_DOWN then
            rowsScroll = rowsScroll + 1
        end
    end

    if down.BUTTON_A then
        HBView.bootHB(selectedHB)
    end
end

function getDimensions()
    local w = hbPerRow * coverWidth + (hbPerRow - 1) * gridGap + 2 * selectorBorder
    local h = 464
    return w, h
end
